;;; keyb.el -- A+ keyboard modifications.

;; Copyright (C) 1998,2001 Colin Rafferty

;; Author: Colin Rafferty <colin@xemacs.org>
;; Keywords: 

;; This program is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; A copy of the GNU General Public License can be obtained from this
;; program's author (send electronic mail to colin@xemacs.org) or from
;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;; Send bug reports to colin@xemacs.org

;;; Code:

(defvar a-key-string nil "Apl characters corresponding to raw keyboard input")

(defvar a-modifier-list '(super)
  "List of modifiers of a key to send the Apl character.")

(defun a-insert-map (akeydef)
  (let ((key (car akeydef))
	(apl (cdr akeydef)))
    (define-key a-minor-map (append a-modifier-list (list key)) 'a-self-insert)
    (define-key a-minor-map (vector '(control c) key) 'a-self-insert)
    (aset a-key-string (char-to-int key) apl)))

(defun a-self-insert (n)
  (interactive "p")
  (or n (setq n 1))
  (let* ((xev (copy-event last-command-event))
	 (key (event-to-character xev t t t)))
    (setq last-command-char (aref a-key-string (char-to-int key)))
    (self-insert-command n)))

(setq a-minor-map (make-sparse-keymap))
(setq a-key-string (make-string 256 ?\000))
(let ((n 32))
  (while (< n 256)
    (aset a-key-string n n)
    (setq n (1+ n))))
(mapcar 'a-insert-map
	'((?a . ?\301) ; A (2)
	  (?b . ?\302) ; B
	  (?c . ?\303) ;
	  (?d . ?\304) ; D
	  (?e . ?\305) ;
	  (?f . ?_)    ; _
	  (?g . ?\307) ; G
	  (?h . ?\310) ; H
	  (?i . ?\311) ;
	  (?j . ?\312) ; J
	  (?k . ?\')   ; '
	  (?l . ?\314) ; L
	  (?m . ?|)    ; |
	  (?n . ?\316) ; N
	  (?o . ?\317) ;
	  (?p . ?\*)   ; *
	  (?q . ?\?)   ; ?
	  (?r . ?\322) ; R
	  (?s . ?\323) ;
	  (?t . ?\~)   ; ~
	  (?u . ?\325) ;
	  (?v . ?\326) ; V
	  (?w . ?\327) ; W (1)
	  (?x . ?\330) ;
	  (?y . ?\331) ;
	  (?z . ?\332) ;
	  (?1 . ?\241) ; !
	  (?2 . ?\242) ;
	  (?3 . ?<)    ; <
	  (?4 . ?\244) ; $
	  (?5 . ?=)    ; =
	  (?6 . ?\246) ;
	  (?7 . ?>)    ; >
	  (?8 . ?\250) ; (
	  (?9 . ?\251) ; )
	  (?0 . ?^)    ; ^
	  (?- . ?\253) ; +
	  (?= . ?\337) ;
	  (?\\ . ?\334);
	  (?` . ?\376) ; ~
	  (?, . ?\254) ; ,
	  (?[ . ?\373) ; {
	  (?] . ?\375) ; }
	  (?\; . ?\333) ; [
	  (?' . ?\335) ;
	  (?. . ?\334) ;
	  (?/ . ?\257) ; /
	  (?! . ?\340) ; `
	  (?@ . ?\346) ;
	  (?# . ?\347) ;
	  (?$ . ?\350) ; h
	  (?% . ?\367) ; w (1)
	  (?^ . ?\364) ; t
	  (?& . ?\341) ; a (2)
	  (?* . ?\360) ; p
	  (?\( . ?\271); 9
	  (?\) . ?\260); 0
	  (?_ . ?!)    ; !
	  (?+ . ?\255) ; -
	  (?| . ?\374) ; | (check syntax table entry!)
	  (?O . ?\357) ;
	  (?J . ?\352) ;
	  (?F . ?\275) ; =
	  (?E . ?\345) ; e
	  (?I . ?\351) ; i
	  (?{ . ?\335) ;
	  (?} . ?\333) ; [
	  (?S . ?\276) ; >
	  (?G . ?\347) ;
	  (?H . ?\350) ; h
	  (?L . ?\354) ; l
	  (?: . ?\274) ;
	  (?\" . ?\273); ;
	  (?C . ?\343) ; c
	  (?B . ?\342) ; b
	  (?N . ?\356) ; n
	  (?M . ?\315) ;
	  (?> . ?\256) ; . *** ???
	  (?P . ?\263) ; 3
	  (?Y . ?\264) ; 4
	  (?Z . ?\372) ; z
	  (?Q . ?\277) ;
	  (?W . ?W)    ; W
	  (?R . ?R)    ; R
	  (?T . ?T)    ; T
	  (?U . ?U)    ; U
	  (?A . ?A)    ; A
	  (?D . ?D)    ; D
	  (?K . ?K)    ; K
	  (?X . ?X)    ; X
	  (?V . ?V)    ; V
	  (?? . ?\?)   ; ?
	  (?~ . ?~)    ; ~
	  (?< . ?<)    ; <
	  (?\040 . ?\040)
	  ) ; end of translation table
	) ; end of mapcar
	  
; if we are running off of a tty, window-system is nil
; in such a case, we need to handle all those events that get passed
; as escape sequences
(if window-system nil
  (global-unset-key "\C-h")
  (global-set-key "\C-h" 'backward-delete-char) 
  (define-key esc-map "[" nil)
  (define-key esc-map "O" nil)
  (mapcar #'(lambda (akeydef)
	      (let ((key (car akeydef))
		    (apl (cdr akeydef)))
		(define-key esc-map key apl)
		))
	  '(
	    ("[224z" . a-send-region)		; f1
	    ("[225z" . a-send-line)		; f2
	    ("[226z" . a-send-func)		; f3
	    ("[227z" . a-other-window)	; f4
	    ("[228z" . delete-other-windows)	; f5
	    ("[229z" . other-window-or-pop)	; f6
	    ("[230z" . next-buffer)		; f7
	    ("[231z" . previous-buffer)	; f8
	    ("[232z" . enlarge-window)	; f9
	    ("[233z" . shrink-window)		; f10
	    ("[209z" . print-buffer)		; PrintScreen
	    ("[210z" . scroll-down-in-place)	; Scroll
	    ("[5z" . scroll-down)		; pgup
	    ("[216z" . scroll-down)		; pgup
	    ("[6z" . scroll-up)		; pgdn
	    ("[222z" . scroll-up)		; pgdn
	    ("[214z" . beginning-of-line)	; home
	    ("[220z" . end-of-line)		; end
	    ("[2z" . overwrite-mode)		; insert
	    ("OA" . previous-line)		; up-arrow
	    ("OB" . next-line)		; down-arrow
	    ("OC" . forward-char)		; right-arrow
	    ("OD" . backward-char)		; left-arrow
	    ) ; end of translation table
	  ) ; end of mapcar
  ) ; end of if window-system nil
(define-key a-minor-map "\C-c\C-a" 'a)
(define-key a-minor-map "\C-c\C-r" 'a-send-region)
(define-key a-minor-map "\C-c\C-l" 'a-send-line)
(define-key a-minor-map "\C-c\C-f" 'a-send-func)
(define-key a-minor-map "\C-c\C-b" 'a-send-buffer)
(define-key a-minor-map '(escape)  esc-map)

(setq a-mode-map nil)
(if a-mode-map nil
  (setq a-mode-map (make-sparse-keymap)))
(define-key a-mode-map "\C-m" 'a-send-input)
(define-key a-mode-map "\C-c\C-c" 'a-interrupt-process)
(define-key a-mode-map "\C-c\C-k" 'a-kill-process)
(define-key a-mode-map "\C-c\C-q" 'a-quit-process)
(define-key a-mode-map "\C-c\C-s" 'a-show-output) 
(define-key a-mode-map "\C-c\C-o" 'a-kill-output) 
(define-key a-mode-map "\C-c\C-p" 'a-goto-last-input)
(define-key a-mode-map "\C-c\C-m" 'a-recall-last-input)

(if (not (assq 'a-minor minor-mode-alist))
    (setq minor-mode-alist (cons '(a-minor " A") minor-mode-alist)))

(provide 'keyb)

;;; keyb.el ends here
